@@ -140,9 +140,6 @@ open(Dirname, Opts) ->
             {error, Reason}
     end.
 
-
-
-
 %% @doc Close a bitcask data store and flush any pending writes to disk.
 -spec close(reference()) -> ok.
 close(Ref) ->
@@ -315,21 +312,27 @@ fold_keys(Ref, Fun, Acc0, MaxAge, MaxPut) ->
 
 %% @doc fold over all K/V pairs in a bitcask datastore.
 %% Fun is expected to take F(K,V,Acc0) -> Acc
--spec fold(reference(), fun((binary(), binary(), any()) -> any()), any()) -> any() | {error, any()}.
-fold(Ref, Fun, Acc0) ->
+-spec fold(reference() | tuple(),
+           fun((binary(), binary(), any()) -> any()), 
+           any()) -> any() | {error, any()}.
+fold(Ref, Fun, Acc0) when is_reference(Ref)->
     State = get_state(Ref),
+    fold(State, Fun, Acc0);
+fold(State, Fun, Acc0) ->
     MaxAge = get_opt(max_fold_age, State#bc_state.opts) * 1000, % convert from ms to us
     MaxPuts = get_opt(max_fold_puts, State#bc_state.opts),
-    fold(Ref, Fun, Acc0, MaxAge, MaxPuts).
+    fold(State, Fun, Acc0, MaxAge, MaxPuts).
 
 %% @doc fold over all K/V pairs in a bitcask datastore specifying max age/updates of
 %% the frozen keystore.
 %% Fun is expected to take F(K,V,Acc0) -> Acc
--spec fold(reference(), fun((binary(), binary(), any()) -> any()), any(),
-          non_neg_integer() | undefined, non_neg_integer() | undefined) -> 
+-spec fold(reference() | tuple(), fun((binary(), binary(), any()) -> any()), any(),
+           non_neg_integer() | undefined, non_neg_integer() | undefined) -> 
                   any() | {error, any()}.
-fold(Ref, Fun, Acc0, MaxAge, MaxPut) ->
+fold(Ref, Fun, Acc0, MaxAge, MaxPut) when is_reference(Ref)->
     State = get_state(Ref),
+    fold(State, Fun, Acc0, MaxAge, MaxPut);
+fold(State, Fun, Acc0, MaxAge, MaxPut) ->
     FrozenFun = 
         fun() ->
                 case open_fold_files(State#bc_state.dirname, 3) of
@@ -456,7 +459,8 @@ merge(Dirname, Opts) ->
 
 %% @doc Merge several data files within a bitcask datastore
 %%      into a more compact form.
--spec merge(Dirname::string(), Opts::[_], FilesToMerge::[string()]) -> ok.
+-spec merge(Dirname::string(), Opts::[_], 
+            {FilesToMerge::[string()],FilesToDelete::[string()]}) -> ok.
 merge(_Dirname, _Opts, []) ->
     ok;
 merge(Dirname,Opts,FilesToMerge) when is_list(FilesToMerge) -> 
@@ -476,7 +480,7 @@ merge(Dirname, Opts, {FilesToMerge0, ExpiredFiles0}) ->
     merge1(Dirname, Opts, FilesToMerge, ExpiredFiles).
 
 %% Inner merge function, assumes that bitcask is running and all files exist.
-merge1(_Dirname, _Opts, [], _ExpiredFiles) ->
+merge1(_Dirname, _Opts, [], []) ->
     ok;
 merge1(Dirname, Opts, FilesToMerge, ExpiredFiles) ->
     %% Test to see if this is a complete or partial merge
@@ -624,7 +628,7 @@ consider_for_merge(FragTrigger, DeadBytesTrigger, ExpirationGraceTime) ->
                        )
     end.
 
--spec needs_merge(reference()) -> {true, [string()]} | false.
+-spec needs_merge(reference()) -> {true, {[string()], [string()]}} | false.
 needs_merge(Ref) ->
     State = get_state(Ref),
     {_KeyCount, Summary} = summary_info(Ref),
@@ -694,7 +698,15 @@ needs_merge(Ref) ->
                     ok
             end,
             FileNames = [Filename || {Filename, _Reasons} <- MergableFiles],
-            ExpiredFiles = [Filename || {Filename, [{data_expired,_,_}]} <- MergableFiles],
+            F = fun(X) ->
+                    case X of
+                        {data_expired,_,_} ->
+                            true;
+                        _ ->
+                            false
+                    end
+            end,
+            ExpiredFiles = [Filename || {Filename, Reasons} <- MergableFiles,  lists:any(F,Reasons)],
             {true, {FileNames, ExpiredFiles}};
         false ->
             false
@@ -868,13 +880,15 @@ scan_key_files([Filename | Rest], KeyDir, Acc, CloseFile, EnoentOK) ->
     %% Restrictive pattern matching below is intentional
     case bitcask_fileops:open_file(Filename) of
         {ok, File} ->
+            FileTstamp = bitcask_fileops:file_tstamp(File),
             F = fun(K, Tstamp, {Offset, TotalSz}, _) ->
                         bitcask_nifs:keydir_put(KeyDir,
                                                 K,
-                                                bitcask_fileops:file_tstamp(File),
+                                                FileTstamp,
                                                 TotalSz,
                                                 Offset,
-                                                Tstamp)
+                                                Tstamp,
+                                                false)
                 end,
             bitcask_fileops:fold_keys(File, F, undefined, recovery),
             if CloseFile == true ->
@@ -1014,7 +1028,7 @@ merge_files(#mstate {  dirname = Dirname,
                        [File#filestate.filename, Dirname, Error]),
                      State
              after
-                    catch bitcask_fileops:close(File)
+                     catch bitcask_fileops:close(File)
              end,
     merge_files(State2#mstate { input_files = Rest }).
 
@@ -1161,6 +1175,7 @@ out_of_date(Key, Tstamp, FileId, {_,_,Offset,_} = Pos, ExpiryTime, EverFound,
             end
     end.
 
+-spec readable_files(string()) -> [string()].  
 readable_files(Dirname) ->
     %% Check the write and/or merge locks to see what files are currently
     %% being written to. Generate our list excepting those.
@@ -1344,8 +1359,17 @@ default_dataset() ->
 %% HACK: Terrible hack to ensure that the .app file for
 %% bitcask is available on the code path. Assumption here
 %% is that we're running in .eunit/ as part of rebar.
+%% MORE HACK: init the io mode (nif or erlang) here.
 a0_test() ->
-    code:add_pathz("../ebin").
+    code:add_pathz("../ebin"),
+    application:start(erlang),
+    Mode0 = case os:getenv("IOMODE") of
+                false -> "erlang";
+                Else -> Else
+            end,
+    Mode = list_to_atom(Mode0),
+    application:set_env(bitcask, io_mode, Mode),
+    error_logger:info_msg("Set IO mode to: ~p", [Mode]).
 
 roundtrip_test() ->
     os:cmd("rm -rf /tmp/bc.test.roundtrip"),
