@@ -49,6 +49,7 @@
          file_pwrite/3,
          file_read/2,
          file_write/2,
+         file_position/2,
          file_seekbof/1]).
 
 -on_load(init/0).
@@ -382,6 +383,13 @@ file_write(Ref, Bytes) ->
 file_write_int(_Ref, _Bytes) ->
     erlang:nif_error({error, not_loaded}).
 
+file_position(Ref, Position) ->
+    bitcask_bump:big(),
+    file_position_int(Ref, Position).
+
+file_position_int(_Ref, _Position) ->
+    erlang:nif_error({error, not_loaded}).
+
 file_seekbof(Ref) ->
     bitcask_bump:big(),
     file_seekbof_int(Ref).
